"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ReplaceUrlHtmlWebpackPlugin = void 0;
const path = require("path");
const HtmlWebpackPlugin = require("html-webpack-plugin");
function replaceJS(html, publicPath, assetTags, compilerOptions) {
    return replace(html, publicPath, assetTags, "script", "src", compilerOptions, /(<script[\S\s]*?src=['"])(.+?)(['"][^>]*?>)/gi);
}
function replaceCSS(html, publicPath, assetTags, compilerOptions) {
    return replace(html, publicPath, assetTags, "link", "href", compilerOptions, /(<link[\S\s]*?href=['"])(.+?)(['"][^>]*?>)/gi);
}
function replace(html, publicPath, assetTags, filterAssetTagName, filterAssetTagAttribute, compilerOptions, regex) {
    const basePath = getBasePath(compilerOptions);
    let output = "";
    let lastIndex = 0;
    let result;
    while ((result = regex.exec(html)) !== null) {
        const scriptPrefix = result[1];
        const scriptSource = result[2];
        const scriptSuffix = result[3];
        output += html.substring(lastIndex, result.index);
        output += scriptPrefix;
        // Resolve script source path
        const resolvedScriptSource = path.join(basePath, scriptSource);
        const scriptSourceName = getPathName(resolvedScriptSource);
        // Determine if source should be replaced
        let replaceFile;
        for (let i = assetTags.length - 1; i >= 0; i--) {
            const assetTag = assetTags[i];
            if (assetTag.tagName !== filterAssetTagName) {
                continue;
            }
            if (!(filterAssetTagAttribute in assetTag.attributes)) {
                continue;
            }
            const assetPath = assetTag.attributes[filterAssetTagAttribute];
            if (typeof assetPath !== "string") {
                continue;
            }
            let assetFileName;
            if (publicPath && assetPath.startsWith(publicPath)) {
                assetFileName = assetPath.slice(publicPath.length);
            }
            else {
                assetFileName = assetPath;
            }
            if (scriptSourceName === getPathName(path.join(basePath, assetFileName))) {
                // Replace!
                replaceFile = assetPath;
                // Remove file from files array
                assetTags.splice(i, 1);
            }
        }
        if (replaceFile != null) {
            output += replaceFile.split(path.win32.sep).join(path.posix.sep);
        }
        else {
            output += scriptSource;
        }
        output += scriptSuffix;
        lastIndex = regex.lastIndex;
    }
    output += html.substring(lastIndex);
    return output;
}
function getBasePath(compilerOptions) {
    let base;
    if (compilerOptions.output && compilerOptions.output.path) {
        base = compilerOptions.output.path;
    }
    else if (compilerOptions.context) {
        base = compilerOptions.context;
    }
    else {
        base = __dirname;
    }
    return base;
}
function getPathName(filePath) {
    let parsedPath = path.parse(filePath);
    const dir = parsedPath.dir;
    while (parsedPath.ext.length > 0) {
        filePath = parsedPath.name;
        parsedPath = path.parse(filePath);
    }
    return path.join(dir, parsedPath.name);
}
const pluginName = "ReplaceUrlHtmlWebpackPlugin";
/**
 *
 */
class ReplaceUrlHtmlWebpackPlugin {
    /**
     * @override
     */
    apply(compiler) {
        const compilerOptions = compiler.options;
        compiler.hooks.compilation.tap(pluginName, (compilation) => {
            let publicPath;
            HtmlWebpackPlugin.getHooks(compilation).beforeAssetTagGeneration.tapAsync(pluginName, (data, cb) => {
                publicPath = data.assets.publicPath;
                cb(null, data);
            });
            HtmlWebpackPlugin.getHooks(compilation).afterTemplateExecution.tapAsync(pluginName, (data, cb) => {
                if (publicPath != null) {
                    // Process both head and body tags
                    for (const assetTags of [data.headTags, data.bodyTags]) {
                        // Replace asset tags in HTML
                        let html = data.html;
                        html = replaceJS(html, publicPath, assetTags, compilerOptions);
                        html = replaceCSS(html, publicPath, assetTags, compilerOptions);
                        // Assign HTML back to data object
                        data.html = html;
                    }
                }
                cb(null, data);
            });
        });
    }
}
exports.ReplaceUrlHtmlWebpackPlugin = ReplaceUrlHtmlWebpackPlugin;
//# sourceMappingURL=index.js.map